#include "..\..\..\Firm_Lib\Ver_120\186COMM.h"

char CmdMsg[512]; //For storing the data of received command
unsigned int CmdLen; //For store the data length of recevie command
unsigned int IsCmdFlag,ErrorFlag,IsHostCommand;
void UserDPRAMIrqFunc(unsigned char INTT) //must be called
{
	int Ret;
	
	if (INTT==SEND_DPRAM_CMD){ //check if commmand is received by using interrupt service routine
		Ret=DPRAMReceiveCmd(CmdMsg,&CmdLen); //Get command information, which is sent from WinPAC, ViewPAC, XPAC, ...
		if (!Ret){
			IsCmdFlag=1; //get a command from host
		}
		else{
			if (Ret!=_NO_DPRAM_CMD) ErrorFlag=Ret; //Trigger the error flag, then show the error message in loop function
		}
	}
}

void UserCANIrqFunc(unsigned char INTT)   //must be called
{
  if (INTT);//Do nothing, avoid the warnning
}

#define SDO_TIMEOUT_VALUE  500 // 100 ms
#define SLAVE_LIFE_TIME   1000 //1000 ms

//define for NodeStatus
#define NO_SLAVE           0x0000
#define SLAVE_BOOT_UP      0x0001
#define SLAVE_IN_LIST      0x0002
#define WAIT_FOR_SDO_REPLY 0x0004
#define SDO_REPLY_ERROR    0x0008

struct {
	unsigned int InitStage;
	unsigned int SDOHeader;
	unsigned int SDOIdx;
	unsigned int SDOSubIdx;
	unsigned int NMTStatus;
	unsigned int NodeStatus;
	unsigned long Obj_1000;
	unsigned long Obj_1018_01;
	unsigned long Obj_1018_02;
	unsigned long Obj_1018_03;
	unsigned long Obj_1018_04;
	COUNTDOWNTIMER SDOTimer;
	COUNTDOWNTIMER LifeTimer;
}SlaveInfo[128];

void UserInitFunc(void)    //must be called
{
	int Ret;
	
	IsCmdFlag=ErrorFlag=IsHostCommand=0;
  	Print("[Dbg]Basic CANopen demo is runnung...\r\n"); //debug information for Debug port
  	Ret=CANConfig(125000UL,0,0,0xffffffffUL,0x00000000UL); //set baud to 125 kbps, and filter is "passing all messages"
  	if (Ret){
  		Print("[Dbg]Config error, error code=%d\r\n",Ret); //debug information for Debug port
  	}
  	EnableSJA1000();
	Print("[Dbg]Run on baud 125 kbps\r\n"); //debug information for Debug port
}



//process the commands from the host
void ProcessHostCmd(void)
{
	int i,ret;

	if (IsCmdFlag){ // check if get commands from host
		if (IsHostCommand){
			DPRAMSendCmd("[Err]Old host command doesn't process yet\r\n",43); //respond command to WinPAC, ViewPAC, XPAC, ...
		}
		else{
			switch (CmdMsg[0]){// check command type
				case 'i':
				case 'I':
					DPRAMSendCmd("[Info]CANopen demo ready. Keyin 'S': scan the network; 'G': Get scan information\r\n",82);
					break;
				case 'S': //the command for scaning the CANopen slaves
				case 's':
					for (i=1;i<128;i++){ //set NodeStatus of all nodes to be initial state, Node 0 is useless
						SlaveInfo[i].NodeStatus=SLAVE_BOOT_UP; //Set slave status in order to into initial stage
						SlaveInfo[i].InitStage=1; //set InitStage to 1 to start initialization procedure
					}
					IsHostCommand=1;
					break;
				case 'G': //the command for showing the information of scanned CANopen slaves
				case 'g':
					for (i=1;i<128;i++){ //Node 0 is useless
						ret=DPRAMWriteMultiByte((i*22),(char *)&SlaveInfo[i].NodeStatus,22);
						if (ret){
							DPRAMSendCmd("[Err]Write data to DPRAM failure\r\n",34); //respond command to WinPAC, ViewPAC, XPAC, ...
							break;	
						}
					}
					if (i==128){ //Write information of all nodes OK 
						DPRAMSendCmd("[Cmd]Data Ready\r\n",17); //respond command to WinPAC, ViewPAC, XPAC, ...
					}
					break;
				default:
					DPRAMSendCmd("[Err]Host command is not support\r\n",34); //respond command to WinPAC, ViewPAC, XPAC, ...
					Print("[Dbg]Host command [0x%02X] is not support\r\n",CmdMsg[0]); //debug information for Debug port
					break;
			}
		}
		IsCmdFlag=0;
	}
}

//Function: SendSDO
//Parameters: NodeID --- slave node ID
//            *Data  --- a data pointer for the sending SDO message to the slave
void SendSDO(unsigned int NodeID, char *Data)
{
	int i,Ret;
	unsigned long MsgID;
	
	MsgID=(0x600+NodeID); // "NodeID is 0" will not happen in this case 
	Ret=SendCANMsg(0,MsgID,0,8,Data);
	if (Ret){
		Print("[Dbg]SendCANMsg error, error code=%d\r\n",Ret); //debug information for Debug port
		SlaveInfo[NodeID].NodeStatus=NO_SLAVE;
		SlaveInfo[NodeID].InitStage=0;
		SlaveInfo[NodeID].NMTStatus=0;
		SlaveInfo[NodeID].SDOHeader=0;
		SlaveInfo[NodeID].SDOIdx=0;
		SlaveInfo[NodeID].SDOSubIdx=0;
		SlaveInfo[NodeID].Obj_1000=0UL;
		SlaveInfo[NodeID].Obj_1018_01=0UL;
		SlaveInfo[NodeID].Obj_1018_02=0UL;
		SlaveInfo[NodeID].Obj_1018_03=0UL;
		SlaveInfo[NodeID].Obj_1018_04=0UL;
		return;
	}
	SlaveInfo[NodeID].SDOHeader=Data[0]; //save the command header in order to check the SDO replied from slave 
	SlaveInfo[NodeID].SDOIdx=*(unsigned int *)&Data[1]; //save the objec index of the SDO command in order to check the SDO replied from slave 
	SlaveInfo[NodeID].SDOSubIdx=Data[3]; //save the object subindex of the SDO command in order to check the SDO replied from slave 
	SlaveInfo[NodeID].NodeStatus=(SlaveInfo[NodeID].NodeStatus&(~SDO_REPLY_ERROR))|WAIT_FOR_SDO_REPLY;
	T_CountDownTimerStart(&SlaveInfo[NodeID].SDOTimer,SDO_TIMEOUT_VALUE); //start to count the SDO timeout
}

//Function: ProcessSDO
//Parameters: NodeID --- slave node ID
//            *Data  --- a data pointer for the data of the received SDO message from the slave
void ProcessSDO(unsigned int NodeID, char *Data)
{
	if (SlaveInfo[NodeID].NodeStatus&WAIT_FOR_SDO_REPLY){ // check if this slave has been sent the SDO command 
		T_CountDownTimerPause(&SlaveInfo[NodeID].SDOTimer); // stop SDO timeout counting
		SlaveInfo[NodeID].NodeStatus&=(~WAIT_FOR_SDO_REPLY);
	}
	if (SlaveInfo[NodeID].SDOHeader==0x40){ //host sends reading SDO command
		if ((SlaveInfo[NodeID].SDOIdx!=(*(unsigned int *)&Data[1])) || (SlaveInfo[NodeID].SDOSubIdx!=Data[3])){ // the reply command doesn't match with the sending command form host
			SlaveInfo[NodeID].NodeStatus|=SDO_REPLY_ERROR;
			if (SlaveInfo[NodeID].InitStage){ //check if initialization is running or not
				SlaveInfo[NodeID].NodeStatus=NO_SLAVE;
				SlaveInfo[NodeID].InitStage=0;
				SlaveInfo[NodeID].NMTStatus=0;
				SlaveInfo[NodeID].SDOHeader=0;
				SlaveInfo[NodeID].SDOIdx=0;
				SlaveInfo[NodeID].SDOSubIdx=0;
				SlaveInfo[NodeID].Obj_1000=0UL;
				SlaveInfo[NodeID].Obj_1018_01=0UL;
				SlaveInfo[NodeID].Obj_1018_02=0UL;
				SlaveInfo[NodeID].Obj_1018_03=0UL;
				SlaveInfo[NodeID].Obj_1018_04=0UL;
			}
			Print("[Dbg]Reply information is not match with the request\r\n"); //debug information for Debug port
			return;
		}
		switch (Data[0]){ //check the header of the SDO message from slaves
			case 0x43: // reply for reading 4-byte SDO
				switch (SlaveInfo[NodeID].SDOIdx){ // check SDO object index
					case 0x1000:
						SlaveInfo[NodeID].Obj_1000=*(unsigned long *)&Data[4]; //save the value of object 0x1000
						if (SlaveInfo[NodeID].InitStage==1) SlaveInfo[NodeID].InitStage=2; //goto next stage
						break; 
					case 0x1018:
						switch(SlaveInfo[NodeID].SDOSubIdx){ // check SDO object subindex
							case 0x00: // for subindex 0
								//do nothing
								break;
							case 0x01: // for subindex 1
								SlaveInfo[NodeID].Obj_1018_01=*(unsigned long *)&Data[4]; //save the value of object 0x1018 subindex 1
								if (SlaveInfo[NodeID].InitStage==2) SlaveInfo[NodeID].InitStage=3; //goto next stage
								break;
							case 0x02: // for subindex 2
								SlaveInfo[NodeID].Obj_1018_02=*(unsigned long *)&Data[4]; //save the value of object 0x1018 subindex 2
								if (SlaveInfo[NodeID].InitStage==3) SlaveInfo[NodeID].InitStage=4; //goto next stage
								break;
							case 0x03: // for subindex 3
								SlaveInfo[NodeID].Obj_1018_03=*(unsigned long *)&Data[4]; //save the value of object 0x1018 subindex 3
								if (SlaveInfo[NodeID].InitStage==4) SlaveInfo[NodeID].InitStage=5; //goto next stage
								break;
							case 0x04: // for subindex 4
								SlaveInfo[NodeID].Obj_1018_04=*(unsigned long *)&Data[4]; //save the value of object 0x1018 subindex 4
								if (SlaveInfo[NodeID].InitStage==5) SlaveInfo[NodeID].InitStage=6; //goto next stage
								break;
							default: // command not support
								SlaveInfo[NodeID].NodeStatus|=SDO_REPLY_ERROR;
								Print("[Dbg]Get an unknow subindex of object 0x1018\r\n"); //debug information for Debug port
								break;
						}
						break;
					default: // object not support
						Print("[Dbg]The object %x is not support\r\n",SlaveInfo[NodeID].SDOIdx); //debug information for Debug port
						break;
				}
				break;
			case 0x4b: // reply for reading 2-byte SDO
				//do nothing
				break;
			case 0x4f: // reply for reading 1-byte SDO
				//do nothing
				break;
			default: // reading SDO error or command not support
		  		SlaveInfo[NodeID].NodeStatus|=SDO_REPLY_ERROR;
		  		Print("[Dbg]Unknow SDO header\r\n"); //debug information for Debug port
		  	break;
		}
	}
	else{ //host sends writting SDO command
		if (Data[0]==0x60){// writting SDO successful
			//do nothing
		}
		else{ //writting SDO fail or command not support
			SlaveInfo[NodeID].NodeStatus|=SDO_REPLY_ERROR;
		}
	}
}

//Check if the node is dead and the SDO transmission is timeout
void RoutineInspection(void)
{
	char SDOData[8]={0x40,0x00,0x10,0x00,0x00,0x00,0x00,0x00}; //get the value of 0x1000 object
	int i;
	for (i=1;i<128;i++){
		if (SlaveInfo[i].NodeStatus==SLAVE_IN_LIST){ //check if node is in list or not
			RefreshWDT();// avoid the WDT timeout
			if (T_CountDownTimerIsTimeUp(&SlaveInfo[i].LifeTimer)){
				SendSDO(i,SDOData); //use "getting the value of 0x1000 object" to confirm if the salve is dead or not
				                    //in many CANopen applications, users may use heartbeat or guarding function to do this job 
				T_CountDownTimerStart(&SlaveInfo[i].LifeTimer,SLAVE_LIFE_TIME); //start to count the LifeTimer timeout
			}
		}
		if (SlaveInfo[i].NodeStatus&WAIT_FOR_SDO_REPLY){ //check if node is waiting for reply or not
			RefreshWDT();// avoid the WDT timeout
			if (T_CountDownTimerIsTimeUp(&SlaveInfo[i].SDOTimer)){ //check if the SDO replied from slave is timeout
				SlaveInfo[i].NodeStatus=NO_SLAVE;
				SlaveInfo[i].InitStage=0;
				SlaveInfo[i].NMTStatus=0;
				SlaveInfo[i].SDOHeader=0;
				SlaveInfo[i].SDOIdx=0;
				SlaveInfo[i].SDOSubIdx=0;
				SlaveInfo[i].Obj_1000=0UL;
				SlaveInfo[i].Obj_1018_01=0UL;
				SlaveInfo[i].Obj_1018_02=0UL;
				SlaveInfo[i].Obj_1018_03=0UL;
				SlaveInfo[i].Obj_1018_04=0UL;
			}
		}
	}
}

//Function: ProcessNMTErrCtrl
//Parameters: NodeID --- slave node ID
//            *Data  --- a data pointer for the data of the received NMT message from the slave
void ProcessNMTErrCtrl(unsigned int NodeID, char *Data)
{
	if (Data[0]){ // if the first byte of CANopen message is not 0
		SlaveInfo[NodeID].NMTStatus=Data[0]&0x7f; // save the NMT status of slave into the NMTStatus value
	}
	else{ //for slave boot-up message
		if (SlaveInfo[NodeID].NodeStatus==NO_SLAVE){ //check if this node is a new-added node or not
			SlaveInfo[NodeID].InitStage=1; // start to initiate it
		}
		SlaveInfo[NodeID].NodeStatus=SLAVE_BOOT_UP; //set node status
		SlaveInfo[NodeID].InitStage=1;
	}
}

//check if the node has been initiated. If not, initiate it.
void InitNode(void){
	char ReadSDOData[8]={0x40,0x00,0x00,0x00,0x00,0x00,0x00,0x00};
	int i,CheckIsCmdRunning=0;
	
	for (i=1;i<128;i++){
		if (SlaveInfo[i].InitStage && (!(SlaveInfo[i].NodeStatus&WAIT_FOR_SDO_REPLY))){
			if (SlaveInfo[i].NodeStatus==SLAVE_BOOT_UP){
				RefreshWDT();// avoid the WDT timeout
				switch(SlaveInfo[i].InitStage){ //check initialization stage of the specific node
					case 1: //get object index 0x1000 subindex 0
						SlaveInfo[i].Obj_1000=0UL; //initiate parameter
						SlaveInfo[i].Obj_1018_01=0UL; //initiate parameter
						SlaveInfo[i].Obj_1018_02=0UL; //initiate parameter
						SlaveInfo[i].Obj_1018_03=0UL; //initiate parameter
						SlaveInfo[i].Obj_1018_04=0UL; //initiate parameter
						ReadSDOData[1]=0x00; //initiate SDO object index
						ReadSDOData[2]=0x10; //initiate SDO object index
						SendSDO(i,ReadSDOData); //Send a SDO message
						break;
					case 2: //get object index 0x1018 subindex 1
						ReadSDOData[1]=0x18; //initiate SDO object index
						ReadSDOData[2]=0x10; //initiate SDO object index
						ReadSDOData[3]=0x1; //initiate SDO object subindex
						SendSDO(i,ReadSDOData); //Send a SDO message
						break;
					case 3: //get object index 0x1018 subindex 2
						ReadSDOData[1]=0x18; //initiate SDO object index
						ReadSDOData[2]=0x10; //initiate SDO object index
						ReadSDOData[3]=0x2; //initiate SDO object subindex
						SendSDO(i,ReadSDOData); //Send a SDO message
						break;
					case 4: //get object index 0x1018 subindex 3
						ReadSDOData[1]=0x18; //initiate SDO object index
						ReadSDOData[2]=0x10; //initiate SDO object index
						ReadSDOData[3]=0x3; //initiate SDO object subindex
						SendSDO(i,ReadSDOData); //Send a SDO message
						break;
					case 5: //get object index 0x1018 subindex 4
						ReadSDOData[1]=0x18; //initiate SDO object index
						ReadSDOData[2]=0x10; //initiate SDO object index
						ReadSDOData[3]=0x4; //initiate SDO object subindex
						SendSDO(i,ReadSDOData); //Send a SDO message
						break;
					case 6: //start to count the salve life time
						T_CountDownTimerStart(&SlaveInfo[i].LifeTimer,SLAVE_LIFE_TIME);
						SlaveInfo[i].NodeStatus=SLAVE_IN_LIST;
						SlaveInfo[i].InitStage=0; //initialization finish
						break;
				}
				if (IsHostCommand) CheckIsCmdRunning=1;
			}
		}
	}
	if (IsHostCommand && (CheckIsCmdRunning==0)){ //The firmware got a host command for initialization of all nodes, and all nodes have been initiated
		DPRAMSendCmd("[Info]Host command is done\r\n",28); //respond command to WinPAC, ViewPAC, XPAC, ...
		IsHostCommand=0;
	}
}

unsigned int LedCnt;
void UserLoopFunc(void)    //must be called
{
	char Data[8],DataLen,Mode,RTR;
	int i,Ret;
	unsigned long MsgID,UpperTime,LowerTime;
	
	
	LedCnt++;
	if (LedCnt&0x400) L2Off(); //flash L2
	else L2On();
	
	RefreshWDT();// avoid the WDT timeout
	ProcessHostCmd();
	RefreshWDT();// avoid the WDT timeout
	InitNode();
	RefreshWDT();// avoid the WDT timeout
	RoutineInspection();
	RefreshWDT();// avoid the WDT timeout
  	Ret=GetCANMsg(&Mode,&MsgID,&RTR,&DataLen,Data,&UpperTime,&LowerTime); //Receiving CAN messages
	if (!Ret){
		if (!Mode){ //check if this CAN message is 2.0A or not
			switch (MsgID>>7){ //check message type
				case 0: //NMT message
					Print("[Dbg]Get a NMT message\r\n"); //debug information for Debug port
					break;
				case 0xb: //SDO (tx) message
					if (MsgID&0x7f){ //check if node ID is valid or not
						ProcessSDO((MsgID&0x7f),Data);
					}
					break;
				case 0xc: //SDO (rx) message
					Print("[Dbg]Get a SDO (rx) message\r\n"); //debug information for Debug port
					break;
				case 0xe: //NMT Error Control message
					if (!RTR){ //check if message comes from other devices or not
						if (MsgID&0x7f){ //check if node ID is valid or not
							ProcessNMTErrCtrl((MsgID&0x7f),Data);
						}
					}
					break;
				default:
					Print("[Dbg]The CANopen message with message ID %lx is not supported\r\n",MsgID); //debug information for Debug port
					break;			
			}
		}
		else{
			Print("[Dbg]Get a CAN 2.0B message\r\n");
		}
	}
	else{
		if (Ret!=_RX_SOFT_BUF_EMPTY){
			Print("[Dbg]Stop to receive data because of receiving error, error code=%d\r\n",Ret); //debug information for Debug port
	 		DisableSJA1000();
		}
	}
	if (ErrorFlag){ // check if get a error from DPRAMReceiveCmd() or not
		Print("[Dbg]DPRAMReceiveCmd error, error code=%d\r\n",ErrorFlag); //debug information for Debug port
  	ErrorFlag=0;
	}
}