#include "..\..\..\Firm_Lib\Ver_120\186COMM.h"

char CmdMsg[512]; //For storing the data of received command
unsigned int CmdLen; //For store the data length of recevie command
unsigned int IsCmdFlag,ErrorFlag,IsHostCommand;
void UserDPRAMIrqFunc(unsigned char INTT) //must be called
{
	int Ret;
	
	if (INTT==SEND_DPRAM_CMD){ //Check if commmand is received by using interrupt service routine
		Ret=DPRAMReceiveCmd(CmdMsg,&CmdLen); //Get command information, which is sent from WinPAC, ViewPAC, XPAC, ...
		if (!Ret){
			IsCmdFlag=1; //get a command from host
		}
		else{
			if (Ret!=_NO_DPRAM_CMD) ErrorFlag=Ret; //Trigger the error flag, then show the error message in loop function
		}
	}
}

void UserCANIrqFunc(unsigned char INTT)   //must be called
{
  if (INTT);//Do nothing, avoid the warnning
}

#define DEVICENET_MASTER_ID  0
#define REPLY_TIMEOUT_VALUE  500 // 100 ms
#define SLAVE_LIFE_TIME   1000 //1000 ms
#define DUPLICATE_TIME    1500 //1500 ms

//define for NodeStatus
#define NO_SLAVE           0x0000
#define ONE_DUPLICATE      0x0001
#define TWO_DUPLICATE      0x0002
#define SLAVE_BOOT_UP      0x0004
#define SLAVE_IN_LIST      0x0008
#define REPLY_MSG          0x0010
#define REPLY_ERROR        0x0020

struct {
	unsigned int InitStage;
	unsigned int NodeStatus;
	unsigned int ConnStatus;
	unsigned int VendorID;
	unsigned long SerialNo;
	unsigned long CmdInfo;
	COUNTDOWNTIMER ReplyTimer;
	COUNTDOWNTIMER LifeTimer;
	COUNTDOWNTIMER DuplicateTimer;
}SlaveInfo[64];

void UserInitFunc(void)    //must be called
{
  	int Ret;
  	IsCmdFlag=ErrorFlag=IsHostCommand=0;
  	Print("[Dbg]Basic DeviceNet demo is runnung...\r\n"); //debug information for Debug port
  	Ret=CANConfig(125000UL,0,0,0xffffffffUL,0x00000000UL); //Set baud to 125 kbps, and filter is "passing all messages"
  	if (Ret){
  		Print("[Dbg]Config error, error code=%d\r\n",Ret); //debug information for Debug port
  	}
  	EnableSJA1000();
  	Print("[Dbg]Run on baud 125 kbps\r\n"); //debug information for Debug port
}

void ProcessHostCmd(void)
{
	int i,ret;

	if (IsCmdFlag){ // check if get commands from host
		if (IsHostCommand){
			DPRAMSendCmd("Old host command doesn't process yet\r\n",38); //respond command to WinPAC, ViewPAC, XPAC, ...
		}
		else{
			switch (CmdMsg[0]){// check command type
				case 'i':
				case 'I':
					DPRAMSendCmd("[Info]DeviceNet Demo ready. Keyin 'S': scan the network; 'G': Get scan information\r\n",84);
					break;
				case 'S': //the command for scaning the CANopen slaves
				case 's':
					for (i=1;i<64;i++){ //set NodeStatus of all nodes to be initial state
						if (i!=DEVICENET_MASTER_ID){
							SlaveInfo[i].NodeStatus=SLAVE_BOOT_UP; //Set slave status in order to into initial stage
							SlaveInfo[i].InitStage=1; //set InitStage to 1 to start initialization procedure
						}
					}
					IsHostCommand=1;
					break;
				case 'G': //the command for showing the information of scanned CANopen slaves
				case 'g':
					for (i=1;i<64;i++){ //Node 0 is useless
						ret=DPRAMWriteMultiByte((i*10),(char *)&SlaveInfo[i].NodeStatus,10);
						if (ret){
							DPRAMSendCmd("[Err]Write data to DPRAM failure\r\n",34); //respond command to WinPAC, ViewPAC, XPAC, ...
							break;	
						}
					}
					if (i==64){ //Write information of all nodes OK 
						DPRAMSendCmd("[Cmd]Data Ready\r\n",17); //respond command to WinPAC, ViewPAC, XPAC, ...
					}
					break;
				default:
					DPRAMSendCmd("[Err]Host command is not support\r\n",34); //respond command to WinPAC, ViewPAC, XPAC, ...
					Print("[Dbg]Host command [0x%02X] is not support\r\n",CmdMsg[0]); //debug information for Debug port
					break;
			}
		}
		IsCmdFlag=0;
	}
}

void UnconnectedExplicitReqMsg(unsigned int NodeID)
{
	unsigned char Data[8]={DEVICENET_MASTER_ID,0x4b,0x03,0x01,0x01,DEVICENET_MASTER_ID,0x00,0x00};
	int i,Ret;
	unsigned long MsgID;
	
	MsgID=(0x400+(NodeID<<3)+0x6); // Group 2 only message ID for unconnected explicit requist message
	
	Ret=SendCANMsg(0,MsgID,0,6,Data);
	if (Ret){
		Print("[Dbg]SendCANMsg error, error code=%d\r\n",Ret); //debug information for Debug port
	}
	SlaveInfo[NodeID].CmdInfo=*(unsigned long *)&Data[1]; //save the command header in order to check the message replied from slave 
	SlaveInfo[NodeID].NodeStatus=(SlaveInfo[NodeID].NodeStatus|REPLY_MSG);
	T_CountDownTimerStart(&SlaveInfo[NodeID].ReplyTimer,REPLY_TIMEOUT_VALUE); //start to count the SDO timeout
}

void ExplicitReqMsg(unsigned int NodeID, unsigned char *Data)
{
	int i,Ret;
	unsigned long MsgID;
	
	MsgID=(0x400+(NodeID<<3)+0x4); // Group 2 only message ID for explicit requist message
	Ret=SendCANMsg(0,MsgID,0,5,Data);
	if (Ret){
		Print("[Dbg]SendCANMsg error, error code=%d\r\n",Ret); //debug information for Debug port
	}
	SlaveInfo[NodeID].CmdInfo=*(unsigned long *)&Data[1]; //save the command header in order to check the message replied from slave 
	SlaveInfo[NodeID].NodeStatus=(SlaveInfo[NodeID].NodeStatus|REPLY_MSG);
	T_CountDownTimerStart(&SlaveInfo[NodeID].ReplyTimer,REPLY_TIMEOUT_VALUE); //start to count the SDO timeout
}

void ProcessGroup2Reply(unsigned int NodeID, unsigned int DLen, char *Data)
{
	int ErrFlag=0;
	
	if (SlaveInfo[NodeID].NodeStatus&REPLY_MSG){ // check if this slave has been sent the SDO command 
		T_CountDownTimerPause(&SlaveInfo[NodeID].ReplyTimer); // stop SDO timeout counting
		SlaveInfo[NodeID].NodeStatus&=(~REPLY_MSG);
	}
	if ((char)((SlaveInfo[NodeID].CmdInfo|0x80)&0xff)!=Data[1]){//check if the response message is valid or not
		Print("[Dbg]Reply information is not match with the request\r\n"); //debug information for Debug port
		ErrFlag=1;
	}
	else{
		SlaveInfo[NodeID].CmdInfo>>=8;
		switch (SlaveInfo[NodeID].CmdInfo){
			case 0x010103UL: //response for unconnected explicit requist message
				if ((DLen!=3) || (Data[2]!=0)){
					Print("[Dbg]Reply connection status error\r\n"); //debug information for Debug port
					ErrFlag=1;
					break;
				}
				if (SlaveInfo[NodeID].InitStage==1) SlaveInfo[NodeID].InitStage=2;
				break;
			case 0x010101UL: //class 01, instance 01, attribute 01 (for vendor ID)
				if (DLen!=4){
					Print("[Dbg]Reply vendor ID error\r\n"); //debug information for Debug port
					ErrFlag=1;
					break;
				}
				SlaveInfo[NodeID].VendorID=*(unsigned int *)&Data[2];
				if (SlaveInfo[NodeID].InitStage==2) SlaveInfo[NodeID].InitStage=3;
				break;
			case 0x060101UL: //class 01, instance 01, attribute 06 (for serial No.)
				if (DLen!=6){
					Print("[Dbg]Reply serial No. error\r\n"); //debug information for Debug port
					ErrFlag=1;
					break;
				}
				SlaveInfo[NodeID].SerialNo=*(unsigned long *)&Data[2];
				if (SlaveInfo[NodeID].InitStage==3) SlaveInfo[NodeID].InitStage=4;
				break;
			case 0x010105UL: //class 05, instance 01, attribute 01 (for connection status)
				if (DLen!=3){
					Print("[Dbg]Reply connection status error\r\n"); //debug information for Debug port
					ErrFlag=1;
					break;
				}
				SlaveInfo[NodeID].ConnStatus=Data[2];
				if (SlaveInfo[NodeID].InitStage==4) SlaveInfo[NodeID].InitStage=5;
				break;
			default:
				Print("[Dbg]Reponse message is not supported\r\n");
				break;
		}
	}
	if (ErrFlag){
		SlaveInfo[NodeID].NodeStatus|=REPLY_ERROR;
		if (SlaveInfo[NodeID].InitStage){ //check if initialization is running or not
			SlaveInfo[NodeID].NodeStatus=NO_SLAVE;
			SlaveInfo[NodeID].InitStage=0;
			SlaveInfo[NodeID].ConnStatus=0;
			SlaveInfo[NodeID].VendorID=0;
			SlaveInfo[NodeID].SerialNo=0;
		}
		return;
	}
}

void RoutineInspection(void)
{
	char ExplicitMsgData[8]={DEVICENET_MASTER_ID,0x0e,0x05,0x01,0x01,0x00,0x00,0x00};
	int i;
	for (i=0;i<64;i++){
		if (i!=DEVICENET_MASTER_ID){
			if (SlaveInfo[i].NodeStatus==SLAVE_IN_LIST){ //check if node is in list or not
				RefreshWDT();// avoid the WDT timeout
				if (T_CountDownTimerIsTimeUp(&SlaveInfo[i].LifeTimer)){
					ExplicitReqMsg(i,ExplicitMsgData); //use "getting the value of connection status" to confirm if the salve is dead or not
					T_CountDownTimerStart(&SlaveInfo[i].LifeTimer,SLAVE_LIFE_TIME); //start to count the LifeTimer timeout
				}
			}
			if (SlaveInfo[i].NodeStatus&(ONE_DUPLICATE|TWO_DUPLICATE)){ //check if receiving duplicate messgae from slaves
				RefreshWDT();// avoid the WDT timeout
				if (SlaveInfo[i].NodeStatus==ONE_DUPLICATE){ //get only one duplicate message
					if (T_CountDownTimerIsTimeUp(&SlaveInfo[i].DuplicateTimer)) SlaveInfo[i].NodeStatus=NO_SLAVE; //if receiving no second duplicate message, set node status
				}
				else{ //get two duplicate message
					if (T_CountDownTimerIsTimeUp(&SlaveInfo[i].DuplicateTimer)){
						SlaveInfo[i].NodeStatus=SLAVE_BOOT_UP; //checking duplicate finish, set node status
						SlaveInfo[i].InitStage=1;
					}
				}
			}
			if (SlaveInfo[i].NodeStatus&REPLY_MSG){ //check if node is waiting for reply or not
				RefreshWDT();// avoid the WDT timeout
				if (T_CountDownTimerIsTimeUp(&SlaveInfo[i].ReplyTimer)){ //check if the reply from slave is timeout
					SlaveInfo[i].NodeStatus=NO_SLAVE;
					SlaveInfo[i].InitStage=0;
					SlaveInfo[i].ConnStatus=0;
					SlaveInfo[i].VendorID=0;
					SlaveInfo[i].SerialNo=0UL;
				}
			}
		}
	}
}

void ProcessDuplicateMsg(unsigned int NodeID, char *Data)
{
	if (Data[0]&0x80){ // the first bit of the first byte to know if this message is sent or replied
		Print("[Dbg]Get a response message of duplicate message from slave %d\r\n",NodeID);
	}
	else{ //for slave boot-up message
		if (SlaveInfo[NodeID].NodeStatus==ONE_DUPLICATE){ //check if this node is a new-added node or not
			SlaveInfo[NodeID].NodeStatus=TWO_DUPLICATE;
		}
		if (SlaveInfo[NodeID].NodeStatus==NO_SLAVE){ //check if this node is a new-added node or not
			SlaveInfo[NodeID].NodeStatus=ONE_DUPLICATE;
		}
		SlaveInfo[NodeID].VendorID=*(unsigned int *)&Data[1];
		SlaveInfo[NodeID].SerialNo=*(unsigned long *)&Data[3];
		T_CountDownTimerStart(&SlaveInfo[NodeID].DuplicateTimer,DUPLICATE_TIME); //start to count the DuplicateTimer timeout
	}
}

void InitNode(void){
	char ExplicitMsgData[8]={DEVICENET_MASTER_ID,0x0e,0x00,0x00,0x00,0x00,0x00,0x00};
	int i,CheckIsCmdRunning=0;
	
	for (i=0;i<64;i++){
		if (i!=DEVICENET_MASTER_ID){
			if (SlaveInfo[i].InitStage && (!(SlaveInfo[i].NodeStatus&REPLY_MSG))){
				if (SlaveInfo[i].NodeStatus==SLAVE_BOOT_UP){
					RefreshWDT();// avoid the WDT timeout
					switch(SlaveInfo[i].InitStage){ //check initialization stage of the specific node
						case 1: //build a connection
							SlaveInfo[i].ConnStatus=0; //initiate parameter
							UnconnectedExplicitReqMsg(i); //Send a unconnected explicit requist message
							break;
						case 2: //get the vendor ID
							ExplicitMsgData[2]=1;
							ExplicitMsgData[3]=1;
							ExplicitMsgData[4]=1;
							ExplicitReqMsg(i,ExplicitMsgData); //Send a explicit requist message to get the connection status
							break;
						case 3: //get the serial No.
							ExplicitMsgData[2]=1;
							ExplicitMsgData[3]=1;
							ExplicitMsgData[4]=6;
							ExplicitReqMsg(i,ExplicitMsgData); //Send a explicit requist message to get the connection status
							break;
						case 4: //get the connection status
							ExplicitMsgData[2]=5;
							ExplicitMsgData[3]=1;
							ExplicitMsgData[4]=1;
							ExplicitReqMsg(i,ExplicitMsgData); //Send a explicit requist message to get the connection status
							break;
						case 5: //start to count the salve life time
							T_CountDownTimerStart(&SlaveInfo[i].LifeTimer,SLAVE_LIFE_TIME);
							SlaveInfo[i].NodeStatus=SLAVE_IN_LIST;
							SlaveInfo[i].InitStage=0; //initialization finish
							break;
					}
					if (IsHostCommand) CheckIsCmdRunning=1;
				}
			}
		}
	}
	if (IsHostCommand && (CheckIsCmdRunning==0)){
		DPRAMSendCmd("[Info]Host command is done\r\n",28); //respond command to WinPAC, ViewPAC, XPAC, ...
		IsHostCommand=0;
	}
}

unsigned int LedCnt;
void UserLoopFunc(void)    //must be called
{
	char Data[8],DataLen,Mode,RTR;
	int i,Ret;
	unsigned long MsgID,UpperTime,LowerTime;
	
	LedCnt++;
	if (LedCnt&0x400) L2Off(); //flash L2
	else L2On();

	RefreshWDT();// avoid the WDT timeout
	ProcessHostCmd();
	RefreshWDT();// avoid the WDT timeout
	InitNode();
	RefreshWDT();// avoid the WDT timeout
	RoutineInspection();
	RefreshWDT();// avoid the WDT timeout
  	Ret=GetCANMsg(&Mode,&MsgID,&RTR,&DataLen,Data,&UpperTime,&LowerTime); //Receiving CAN messages
	if (!Ret){
		if (!Mode){ //check if this CAN message is 2.0A or not
			if ((MsgID&0x600)==0x400){ //for Group 2 messages
				MsgID&=0x1ff;
				switch (MsgID&0x7){ //check the message type
					case 3: //for explicit message reply
						ProcessGroup2Reply((MsgID>>3),DataLen,Data);
						break;
					case 7: //for duplicate message
						ProcessDuplicateMsg((MsgID>>3),Data);
						break;
					default:
						Print("[Dbg]Unsupport Group 2 message, MsgID=%lx\r\n", MsgID);
						break;
				}
			}
			else{ //for Group 1, Group 3, and Group 4 messages
				if (MsgID&0x400){
					Print("[Dbg]Get Group 3 or 4 messages\r\n");
				}
				else{
					Print("[Dbg]Get Group 1 messages\r\n");
				}
			}
		}
		else{
			Print("[Dbg]Get a CAN 2.0B message\r\n");
		}
	}
	else{
		if (Ret!=_RX_SOFT_BUF_EMPTY){
			Print("[Dbg]Stop to receive data because of receiving error, error code=%d\r\n",Ret); //debug information for Debug port
	 		DisableSJA1000();
		}
	}
	if (ErrorFlag){ // check if get a error from DPRAMReceiveCmd() or not
		Print("[Dbg]DPRAMReceiveCmd error, error code=%d\r\n",ErrorFlag); //debug information for Debug port
  	ErrorFlag=0;
	}
}