﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using I8120net_PAC;


using System.Runtime.InteropServices;

namespace RW_DPRAM
{
    public partial class Form1 : Form
    {
        static byte Slot=1;
        static string I8120W_Info, StrText;
        I8120.UserISR UserEvent= new I8120.UserISR(MyDPRAMIntt);
        static Object ThisLock = new Object(); //This is used to avoid mulit-accessing the same I-8120W
                                                //in the different functions or threads simultaneously. It is useless 
                                                //while the functions or threads access the I-8120W with different slot.

        static void MyDPRAMIntt()
        {
            byte [] DataBuf = new byte[640];
            byte InttValue;
            int Ret,i;
            ushort DataLen,NodeStatus;

            I8120.I8120_ReadUserInttType(Slot, out InttValue);
            if (InttValue == I8120.CAN_COMM_CMD_FROM_I8120) //Check if the main control unit got any command from the I-8120W
            {
                lock (ThisLock)
                {
                    //This is used to avoid mulit-accessing the same I-8120W
                    //in the different functions or threads simultaneously. It is useless 
                    //while the functions or threads access the I-8120W with different slot.
                    Ret = I8120.I8120_ReceiveCmd(Slot, DataBuf, out DataLen); //Get command from I-8120W
                }
                if (Ret != 0)
                {
                    if (Ret != I8120.I8120_NoDpramCmd)         //Error happen
                    {
                        I8120W_Info = "Recevie command from I-8120W fail, error code " + Ret.ToString() + "\r\n";
                    }
                }
                else
                {
                    if (DataBuf[1] == 'C') //process [Cmd] messages from the user-defined firmware
                    {
                        lock (ThisLock)
                        {
                            Ret = I8120.I8120_DPRAMReadMultiByte(Slot, 0, DataBuf, 2816); //Get node information from DPRAM 
                        }
                        if (Ret != 0) //Error happen
                        {
                            I8120W_Info = "Read DPRAM from I-8120W fail, error code " + Ret.ToString() + "\r\n";
                        }
                        else
                        {
                            Ret = 1; //Use this for recording if there is any node scanned by the I-8120W user-defined firmware
                            for (i = 1; i < 64; i++)
                            {
                                NodeStatus = DataBuf[i * 10];
                                if (NodeStatus != 0) //check if the node exists or not
                                {
                                    Ret = 0;
                                    I8120W_Info = I8120W_Info + "Node[" + i.ToString("D3") + "]: NodeStatus[" + NodeStatus.ToString("X2") +
                                                  "], CommStatus[" + BitConverter.ToUInt16(DataBuf, i*10+2).ToString("X4") + "], VID["+
                                                  BitConverter.ToUInt16(DataBuf, i*10+4).ToString("X4") + "], SN["+
                                                  BitConverter.ToUInt32(DataBuf, i*10+6).ToString("X8") + "]\r\n";
                                }
                            }
                            if (Ret==1)
                            {
                                I8120W_Info = "No DeviceNet node is detected!\r\n";
                            }
                        }

                    }
                    else //process [Info] messages and [Err] messages from the user-defined firmware
                    {
                        if (DataLen < 512)
                        {
                            DataBuf[DataLen]=0; //End the char array
                            I8120W_Info = I8120W_Info + System.Text.ASCIIEncoding.ASCII.GetString(DataBuf, 0, DataLen) + "\r\n"; //Save the received command to the variable I8120W_Info
                        }

                    }
                }
			}
        }

        public Form1()
        {
            InitializeComponent();
        }

        private void Btn_Init_Click(object sender, EventArgs e)
        {
            int Ret,RetryCnt;
            byte[] SendBuf = new byte[10];
            byte[] RecBuf = new byte[512];
            ushort RecLen;

            lock (ThisLock)
            {
                Ret = I8120.I8120_Init(Slot); //Initiate I-8120W
            }

            if (Ret != 0)
            {
                StrText = String.Format("I-8120 Init Err = {0:D}", Ret);
                MessageBox.Show(StrText);
                return;
            }

            lock (ThisLock)
            {
                Ret = I8120.I8120_HardwareReset(Slot); //Reset the hardware of the I-8120W    
            }
                
            if (Ret != 0)    
            {
                StrText = String.Format("I-8120 HardwareReset Err = {0:D}", Ret);
                MessageBox.Show(StrText);
                return;
            }


            RetryCnt = 5; //Retry 5 times to wait the completion of the initialization of the I-8120W user-defined firmware
            SendBuf[0] = 0x69; //Send cmd "i" to the I-8120W user-defined firmware to check if the firmware is ready or not
            while (RetryCnt-- > 0)
            {
                lock (ThisLock)
                {
                    Ret = I8120.I8120_SendReceiveCmd(Slot, 1, SendBuf, 1, RecBuf, out RecLen, 1000);
                    //get the reply by using the polling method. This function must be call before using the user ISR routine.
                    //Otherwise, the reply may be anticipated by the I8120_ReceiveCmd of the user ISR routine. Then, this function 
                    //will get no drpam data and return the error code.
                }

                if (Ret == 0)
                {
                    if (RecLen < 512)
                    {
                        RecBuf[RecLen] = 0; //End the char array
                        Text_Receive.Text = System.Text.ASCIIEncoding.ASCII.GetString(RecBuf, 0, RecLen) + "\r\n"; //Show the reply from the I-8120W
                        break;
                    }
                }
            }

            if (RetryCnt == 0)
            {
                StrText = String.Format("Check the firmware initialization of the slot {0:D} fail", Ret);
                MessageBox.Show(StrText);
                lock (ThisLock)
                {
                    I8120.I8120_RestoreI8120(Slot); //Recover the system resource
                }
                return;

            }

            lock (ThisLock)
            {
                Ret = I8120.I8120_InstallUserISR(Slot, UserEvent); //Install the user's ISR routine to get the reply by using ISR routine
            }

            timer1.Enabled = true;
            this.Text = "DeviceNet Demo For User-defined Firmware at Slot " + Slot.ToString(); //Modify the title of the dialog
            Btn_Init.Enabled = false;
            Btn_Send.Enabled = true;
            Btn_Clear1.Enabled = true;
            Btn_Clear2.Enabled = true;
        }

        private void Btn_Clear2_Click(object sender, EventArgs e)
        {
            Text_Receive.Text = "";
        }

        private void Combo_Slot_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (Slot != Convert.ToByte(Combo_Slot.SelectedIndex+1))
            {
                lock (ThisLock)
                {
                    I8120.I8120_RestoreI8120(Slot); //Recover the system resource
                }
                Slot = Convert.ToByte(Combo_Slot.SelectedIndex+1);
                timer1.Enabled = false;
                this.Text = "DeviceNet Demo For User-defined Firmware"; //Modify the title of the dialog
                Btn_Init.Enabled = true;
                Btn_Send.Enabled = false;
                Btn_Clear1.Enabled = false;
                Btn_Clear2.Enabled = false;

            }

        }

        private void Btn_Clear1_Click(object sender, EventArgs e)
        {
            Text_Send.Text = "";
        }

        private void Btn_Send_Click(object sender, EventArgs e)
        {

            byte[] SendBuf = new byte[512], ReceiveBuf = new byte[512];
            int Ret;
            ushort StringLength;


            SendBuf = System.Text.ASCIIEncoding.ASCII.GetBytes(Text_Send.Text); //Get commands sent to I-8120W from Edit box
            StringLength = Convert.ToUInt16(Text_Send.Text.Length);
            
            if (StringLength == 0) return;

            lock (ThisLock)
            {
                Ret = I8120.I8120_SendCmd(Slot, SendBuf, StringLength); //Send command to I-8120W
            }
            if (Ret != 0)
            {
                StrText = String.Format("Send command to I-8120W fail, error code = {0:D}", Ret);
                MessageBox.Show(StrText);
            }
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            Btn_Init.Enabled = true;

            Btn_Send.Enabled = false;
            Btn_Clear1.Enabled = false;
            Btn_Clear2.Enabled = false;

            Combo_Slot.SelectedIndex = 0;
            Slot = 1;

            I8120W_Info = "";
        }

        private void Form1_Closed(object sender, EventArgs e)
        {
            lock (ThisLock)
            {
                I8120.I8120_RestoreI8120(Slot); //Recover the system resource
            }
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            if (I8120W_Info.Length != 0) //Check if any command stored in the variable I8120_Info
            {
                Text_Receive.Text = I8120W_Info; //Put the command into the text box
                I8120W_Info = "";
            }
        }
    }
}