Imports I8120net_PAC

Public Class Form1

    Dim Slot As Byte
    Dim I8120W_Info As String, StrText As String
    Dim MyIntt = New I8120.UserISR(AddressOf MyDPRAMIntt)

    Sub MyDPRAMIntt()
        Dim DataBuf(630) As Byte, InttValue As Byte
        Dim Ret As Integer
        Dim DataLen As UShort

        I8120.I8120_ReadUserInttType(Slot, InttValue)
        If (InttValue = I8120.CAN_COMM_CMD_FROM_I8120) Then 'Check if the main control unit got any command from the I-8120W
            SyncLock Me
                'This is used to avoid mulit-accessing the same I-8120W
                'in the different functions or threads simultaneously. It is useless 
                'while the functions or threads access the I-8120W with different slot.
                Ret = I8120.I8120_ReceiveCmd(Slot, DataBuf, DataLen) 'Get command from I-8120W
            End SyncLock
            If (Ret <> 0) Then
                If (Ret <> I8120.I8120_NoDpramCmd) Then 'Error happen
                    I8120W_Info = "Recevie command from I-8120W fail, error code " + Ret.ToString() + vbNewLine
                End If
            Else
                If (DataLen < 512) Then
                    DataBuf(DataLen) = 0 'End the char array
                    I8120W_Info = I8120W_Info + System.Text.ASCIIEncoding.ASCII.GetString(DataBuf, 0, DataLen) + vbNewLine 'Save the received command to the variable I8120W_Info
                End If
            End If
        End If
    End Sub

    Private Sub Combo_Slot_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Combo_Slot.SelectedIndexChanged
        If (Slot <> Combo_Slot.SelectedIndex + 1) Then
            SyncLock Me
                I8120.I8120_RestoreI8120(Slot) 'Recover the system resource
            End SyncLock
            Slot = Combo_Slot.SelectedIndex + 1
            Timer1.Enabled = False
            Me.Text = "RxMsg Demo For User-defined Firmware" 'Modify the title of the dialog
            Btn_Init.Enabled = True
            Btn_Send.Enabled = False
            Btn_Clear1.Enabled = False
            Btn_Clear2.Enabled = False
        End If
    End Sub
    Private Sub Btn_Init_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Btn_Init.Click
        Dim Ret As Integer, RetryCnt As Integer
        Dim SendBuf(10) As Byte
        Dim RecBuf(512) As Byte
        Dim RecLen As UShort

        SyncLock Me
            Ret = I8120.I8120_Init(Slot) 'Initiate I-8120W
        End SyncLock
        If (Ret <> 0) Then
            MessageBox.Show("I-8120 Init Error, Error Code = " + Str(Ret))
            Exit Sub
        End If

        SyncLock Me
            Ret = I8120.I8120_HardwareReset(Slot) 'Reset the hardware of the I-8120W
        End SyncLock
        If (Ret <> 0) Then
            MessageBox.Show("I-8120 HardwareReset Error, Error Code = " + Str(Ret))
            Exit Sub
        End If

        RetryCnt = 5 'Retry 5 times to wait the completion of the initialization of the I-8120W user-defined firmware
        SendBuf(0) = Convert.ToByte(&H49) 'Send cmd "i" to the I-8120W user-defined firmware to check if the firmware is ready or not
        Do While RetryCnt > 0
            SyncLock Me
                Ret = I8120.I8120_SendReceiveCmd(Slot, 1, SendBuf, 1, RecBuf, RecLen, 1000)
                'get the reply by using the polling method. This function must be call before using the user ISR routine.
                'Otherwise, the reply may be anticipated by the I8120_ReceiveCmd of the user ISR routine. Then, this function 
                'will get no drpam data and return the error code.
            End SyncLock

            If (Ret = 0) Then
                If (RecLen < 512) Then
                    RecBuf(RecLen) = 0 'End the char array
                    Text_Receive.Text = System.Text.ASCIIEncoding.ASCII.GetString(RecBuf, 0, RecLen) + vbNewLine 'Show the reply from the I-8120W
                    Exit Do
                End If
            End If
        Loop

        If (RetryCnt = 0) Then
            MessageBox.Show("Check the firmware initialization of the slot " + Str(Slot) + " fail")
            SyncLock Me
                I8120.I8120_RestoreI8120(Slot) 'Recover the system resource
            End SyncLock
            Exit Sub
        End If

        SyncLock Me
            Ret = I8120.I8120_InstallUserISR(Slot, MyIntt) 'Install the user's ISR routine
        End SyncLock

        Timer1.Enabled = True

        Me.Text = "RxMsg Demo For User-defined Firmware at Slot " + Slot.ToString() 'Modify the title of the dialog
        Btn_Init.Enabled = False
        Btn_Send.Enabled = True
        Btn_Clear1.Enabled = True
        Btn_Clear2.Enabled = True
    End Sub

    Private Sub Form1_Closed(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.Closed
        SyncLock Me
            I8120.I8120_RestoreI8120(Slot) 'Recover the system resource
        End SyncLock
    End Sub

    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        Btn_Init.Enabled = True

        Btn_Send.Enabled = False
        Btn_Clear1.Enabled = False
        Btn_Clear2.Enabled = False

        Combo_Slot.SelectedIndex = 0
        Slot = 1

        I8120W_Info = ""
    End Sub

    Private Sub Timer1_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Timer1.Tick
        If (I8120W_Info.Length() <> 0) Then 'Check if any command stored in the variable I8120_Info
            Text_Receive.Text = I8120W_Info 'Put the command into the text box
            I8120W_Info = ""
        End If
    End Sub

    Private Sub Btn_Send_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Btn_Send.Click
        Dim SendBuf(100) As Byte
        Dim Ret As Integer, StringLength As Integer

        SendBuf = System.Text.ASCIIEncoding.ASCII.GetBytes(Text_Send.Text) 'Get commands sent to I-8120W from Edit box
        StringLength = Text_Send.Text.Length

        If (StringLength = 0) Then Exit Sub

        SyncLock Me
            Ret = I8120.I8120_SendCmd(Slot, SendBuf, StringLength) 'send command to I-8120W
        End SyncLock
        If (Ret <> 0) Then
            MessageBox.Show("Send command to I-8120W Error, Error Code = " + Str(Ret))
            Exit Sub
        End If
    End Sub

    Private Sub Btn_Clear1_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Btn_Clear1.Click
        Text_Send.Text = ""
    End Sub

    Private Sub Btn_Clear2_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Btn_Clear2.Click
        Text_Receive.Text = ""
    End Sub
End Class

